@@ -32,9 +32,11 @@
 #include <curl/curl.h>
 #include <json/json.h>
 #include <ctime>
+#include <functional>
 #include <fstream>
 #include <deque>
 
+class cloudSaveFile;
 class Timer
 {
     public:
@@ -94,11 +96,18 @@ class Downloader
         int init();
         int login();
         int listGames();
-        int listTags();
         void checkNotifications();
         void clearUpdateNotifications();
         void repair();
         void download();
+
+        void downloadCloudSaves(const std::string& product_id, int build_index = -1);
+        void downloadCloudSavesById(const std::string& product_id, int build_index = -1);
+        void uploadCloudSaves(const std::string& product_id, int build_index = -1);
+        void uploadCloudSavesById(const std::string& product_id, int build_index = -1);
+        void deleteCloudSaves(const std::string& product_id, int build_index = -1);
+        void deleteCloudSavesById(const std::string& product_id, int build_index = -1);
+
         void checkOrphans();
         void checkStatus();
         void updateCache();
@@ -113,9 +122,16 @@ class Downloader
         void galaxyInstallGame(const std::string& product_id, int build_index = -1, const unsigned int& iGalaxyArch = GlobalConstants::ARCH_X64);
         void galaxyInstallGameById(const std::string& product_id, int build_index = -1, const unsigned int& iGalaxyArch = GlobalConstants::ARCH_X64);
         void galaxyShowBuilds(const std::string& product_id, int build_index = -1);
+        void galaxyShowCloudSaves(const std::string& product_id, int build_index = -1);
+        void galaxyShowLocalCloudSaves(const std::string& product_id, int build_index = -1);
+        void galaxyShowLocalCloudSavesById(const std::string& product_id, int build_index = -1);
         void galaxyShowBuildsById(const std::string& product_id, int build_index = -1);
+        void galaxyShowCloudSavesById(const std::string& product_id, int build_index = -1);
     protected:
     private:
+        std::map<std::string, std::string> cloudSaveLocations(const std::string& product_id, int build_index);
+        int cloudSaveListByIdForEach(const std::string& product_id, int build_index, const std::function<void(cloudSaveFile &)> &f);
+
         CURLcode downloadFile(const std::string& url, const std::string& filepath, const std::string& xml_data = std::string(), const std::string& gamename = std::string());
         int repairFile(const std::string& url, const std::string& filepath, const std::string& xml_data = std::string(), const std::string& gamename = std::string());
         int getGameDetails();
@@ -125,6 +141,7 @@ class Downloader
         std::string getResponse(const std::string& url);
         std::string getLocalFileHash(const std::string& filepath, const std::string& gamename = std::string());
         std::string getRemoteFileHash(const gameFile& gf);
+        void addStatusLine(const std::string& statusCode, const std::string& gamename, const std::string& filepath, const uintmax_t& filesize, const std::string& localHash);
         int loadGameDetailsCache();
         int saveGameDetailsCache();
         std::vector<gameDetails> getGameDetailsFromJsonNode(Json::Value root, const int& recursion_level = 0);
@@ -133,9 +150,12 @@ class Downloader
         static std::string getChangelogFromJSON(const Json::Value& json);
         void saveChangelog(const std::string& changelog, const std::string& filepath);
         static void processDownloadQueue(Config conf, const unsigned int& tid);
+        static void processCloudSaveDownloadQueue(Config conf, const unsigned int& tid);
+        static void processCloudSaveUploadQueue(Config conf, const unsigned int& tid);
         static int progressCallbackForThread(void *clientp, curl_off_t dltotal, curl_off_t dlnow, curl_off_t ultotal, curl_off_t ulnow);
         template <typename T> void printProgress(const ThreadSafeQueue<T>& download_queue);
         static void getGameDetailsThread(Config config, const unsigned int& tid);
+        void printGameDetailsAsText(gameDetails& game);
 
         static int progressCallback(void *clientp, curl_off_t dltotal, curl_off_t dlnow, curl_off_t ultotal, curl_off_t ulnow);
         static size_t writeData(void *ptr, size_t size, size_t nmemb, FILE *stream);
