@@ -24,6 +24,18 @@ template<typename T> void set_vm_value(std::map<std::string, bpo::variable_value
     vm[option].value() = boost::any(value);
 }
 
+void ensure_trailing_slash(std::string &path, const char *default_ = nullptr) {
+    if (!path.empty())
+    {
+        if (path.at(path.length()-1)!='/')
+            path += "/";
+    }
+    else
+    {
+        path = default_; // Directory wasn't specified, use current directory
+    }
+}
+
 int main(int argc, char *argv[])
 {
     struct sigaction act;
@@ -149,8 +161,20 @@ int main(int argc, char *argv[])
     }
     include_options_text += "Separate with \",\" to use multiple values";
 
+    // Create help text for --list-format option
+    std::string list_format_text = "List games/tags\n";
+    for (unsigned int i = 0; i < GlobalConstants::LIST_FORMAT.size(); ++i)
+    {
+        list_format_text += GlobalConstants::LIST_FORMAT[i].str + " = " + GlobalConstants::LIST_FORMAT[i].regexp + "\n";
+    }
+
     std::string galaxy_product_id_install;
     std::string galaxy_product_id_show_builds;
+    std::string galaxy_product_id_show_cloud_paths;
+    std::string galaxy_product_id_show_local_cloud_paths;
+    std::string galaxy_product_cloud_saves;
+    std::string galaxy_product_cloud_saves_delete;
+    std::string galaxy_upload_product_cloud_saves;
     std::string tags;
 
     std::vector<std::string> vFileIdStrings;
@@ -166,6 +190,7 @@ int main(int argc, char *argv[])
     bpo::options_description options_cfg_only;
     bpo::options_description options_cfg_all("Configuration");
     bool bClearUpdateNotifications = false;
+    bool bList = false;
     try
     {
         bool bInsecure = false;
@@ -185,14 +210,14 @@ int main(int argc, char *argv[])
         std::string sGalaxyLanguage;
         std::string sGalaxyArch;
         std::string sGalaxyCDN;
+        std::string sListFormat;
         Globals::globalConfig.bReport = false;
         // Commandline options (no config file)
         options_cli_no_cfg.add_options()
             ("help,h", "Print help message")
             ("version", "Print version information")
             ("login", bpo::value<bool>(&Globals::globalConfig.bLogin)->zero_tokens()->default_value(false), "Login")
-            ("list", bpo::value<bool>(&Globals::globalConfig.bList)->zero_tokens()->default_value(false), "List games")
-            ("list-details", bpo::value<bool>(&Globals::globalConfig.bListDetails)->zero_tokens()->default_value(false), "List games with detailed info")
+            ("list", bpo::value<std::string>(&sListFormat)->implicit_value("games"), list_format_text.c_str())
             ("download", bpo::value<bool>(&Globals::globalConfig.bDownload)->zero_tokens()->default_value(false), "Download")
             ("repair", bpo::value<bool>(&Globals::globalConfig.bRepair)->zero_tokens()->default_value(false), "Repair downloaded files\nUse --repair --download to redownload files when filesizes don't match (possibly different version). Redownload will rename the old file (appends .old to filename)")
             ("game", bpo::value<std::string>(&Globals::globalConfig.sGameRegex)->default_value(""), "Set regular expression filter\nfor download/list/repair (Perl syntax)")
@@ -213,10 +238,14 @@ int main(int argc, char *argv[])
             ("cacert", bpo::value<std::string>(&Globals::globalConfig.curlConf.sCACertPath)->default_value(""), "Path to CA certificate bundle in PEM format")
             ("respect-umask", bpo::value<bool>(&Globals::globalConfig.bRespectUmask)->zero_tokens()->default_value(false), "Do not adjust permissions of sensitive files")
             ("user-agent", bpo::value<std::string>(&Globals::globalConfig.curlConf.sUserAgent)->default_value(DEFAULT_USER_AGENT), "Set user agent")
+
+            ("wine-prefix", bpo::value<std::string>(&Globals::globalConfig.dirConf.sWinePrefix)->default_value("."), "Set wineprefix directory")
+            ("cloud-whitelist", bpo::value<std::vector<std::string>>(&Globals::globalConfig.cloudWhiteList)->multitoken(), "Include this list of cloud saves, by default all cloud saves are included\n Example: --cloud-whitelist saves/AutoSave-0 saves/AutoSave-1/screenshot.png")
+            ("cloud-blacklist", bpo::value<std::vector<std::string>>(&Globals::globalConfig.cloudBlackList)->multitoken(), "Exclude this list of cloud saves\n Example: --cloud-blacklist saves/AutoSave-0 saves/AutoSave-1/screenshot.png")
+            ("cloud-force", bpo::value<bool>(&Globals::globalConfig.bCloudForce)->zero_tokens()->default_value(false), "Download or Upload cloud saves even if they're up-to-date\nDelete remote cloud saves even if no saves are whitelisted")
 #ifdef USE_QT_GUI_LOGIN
             ("enable-login-gui", bpo::value<bool>(&Globals::globalConfig.bEnableLoginGUI)->zero_tokens()->default_value(false), "Enable login GUI when encountering reCAPTCHA on login form")
 #endif
-            ("list-tags", bpo::value<bool>(&Globals::globalConfig.bListTags)->zero_tokens()->default_value(false), "List tags")
             ("tag", bpo::value<std::string>(&tags)->default_value(""), "Filter using tags. Separate with \",\" to use multiple values")
         ;
         // Commandline options (config file)
@@ -232,7 +261,6 @@ int main(int argc, char *argv[])
             ("no-color", bpo::value<bool>(&bNoColor)->zero_tokens()->default_value(false), "Don't use coloring in the progress bar or status messages")
             ("no-duplicate-handling", bpo::value<bool>(&bNoDuplicateHandler)->zero_tokens()->default_value(false), "Don't use duplicate handler for installers\nDuplicate installers from different languages are handled separately")
             ("no-subdirectories", bpo::value<bool>(&bNoSubDirectories)->zero_tokens()->default_value(false), "Don't create subdirectories for extras, patches and language packs")
-            ("verbose", bpo::value<bool>(&Globals::globalConfig.bVerbose)->zero_tokens()->default_value(false), "Print lots of information")
             ("curl-verbose", bpo::value<bool>(&Globals::globalConfig.curlConf.bVerbose)->zero_tokens()->default_value(false), "Set libcurl to verbose mode")
             ("insecure", bpo::value<bool>(&bInsecure)->zero_tokens()->default_value(false), "Don't verify authenticity of SSL certificates")
             ("timeout", bpo::value<long int>(&Globals::globalConfig.curlConf.iTimeout)->default_value(10), "Set timeout for connection\nMaximum time in seconds that connection phase is allowed to take")
@@ -261,6 +289,7 @@ int main(int argc, char *argv[])
             ("lowspeed-rate", bpo::value<long int>(&Globals::globalConfig.curlConf.iLowSpeedTimeoutRate)->default_value(200), "Set average transfer speed in bytes per second that the transfer should be below during time specified with --lowspeed-timeout for it to be considered too slow and aborted")
             ("include-hidden-products", bpo::value<bool>(&Globals::globalConfig.bIncludeHiddenProducts)->zero_tokens()->default_value(false), "Include games that have been set hidden in account page")
             ("size-only", bpo::value<bool>(&Globals::globalConfig.bSizeOnly)->zero_tokens()->default_value(false), "Don't check the hashes of the files whose size matches that on the server")
+            ("verbosity", bpo::value<int>(&Globals::globalConfig.iMsgLevel)->default_value(0), "Set message verbosity level\n -1 = Less verbose\n 0 = Default\n 1 = Verbose\n 2 = Debug")
         ;
 
         options_cli_no_cfg_hidden.add_options()
@@ -271,6 +300,11 @@ int main(int argc, char *argv[])
         options_cli_experimental.add_options()
             ("galaxy-install", bpo::value<std::string>(&galaxy_product_id_install)->default_value(""), "Install game using product id [product_id/build_index] or gamename regex [gamename/build_id]\nBuild index is used to select a build and defaults to 0 if not specified.\n\nExample: 12345/2 selects build 2 for product 12345")
             ("galaxy-show-builds", bpo::value<std::string>(&galaxy_product_id_show_builds)->default_value(""), "Show game builds using product id [product_id/build_index] or gamename regex [gamename/build_id]\nBuild index is used to select a build and defaults to 0 if not specified.\n\nExample: 12345/2 selects build 2 for product 12345")
+            ("galaxy-download-cloud-saves", bpo::value<std::string>(&galaxy_product_cloud_saves)->default_value(""), "Download cloud saves using product-id [product_id/build_index] or gamename regex [gamename/build_id]\nBuild index is used to select a build and defaults to 0 if not specified.\n\nExample: 12345/2 selects build 2 for product 12345")
+            ("galaxy-upload-cloud-saves", bpo::value<std::string>(&galaxy_upload_product_cloud_saves)->default_value(""), "Upload cloud saves using product-id [product_id/build_index] or gamename regex [gamename/build_id]\nBuild index is used to select a build and defaults to 0 if not specified.\n\nExample: 12345/2 selects build 2 for product 12345")
+            ("galaxy-show-cloud-saves", bpo::value<std::string>(&galaxy_product_id_show_cloud_paths)->default_value(""), "Show game cloud-saves using product id [product_id/build_index] or gamename regex [gamename/build_id]\nBuild index is used to select a build and defaults to 0 if not specified.\n\nExample: 12345/2 selects build 2 for product 12345")
+            ("galaxy-show-local-cloud-saves", bpo::value<std::string>(&galaxy_product_id_show_local_cloud_paths)->default_value(""), "Show local cloud-saves using product id [product_id/build_index] or gamename regex [gamename/build_id]\nBuild index is used to select a build and defaults to 0 if not specified.\n\nExample: 12345/2 selects build 2 for product 12345")
+            ("galaxy-delete-cloud-saves", bpo::value<std::string>(&galaxy_product_cloud_saves_delete)->default_value(""), "Delete cloud-saves using product id [product_id/build_index] or gamename regex [gamename/build_id]\nBuild index is used to select a build and defaults to 0 if not specified.\n\nExample: 12345/2 selects build 2 for product 12345")
             ("galaxy-platform", bpo::value<std::string>(&sGalaxyPlatform)->default_value("w"), galaxy_platform_text.c_str())
             ("galaxy-language", bpo::value<std::string>(&sGalaxyLanguage)->default_value("en"), galaxy_language_text.c_str())
             ("galaxy-arch", bpo::value<std::string>(&sGalaxyArch)->default_value("x64"), galaxy_arch_text.c_str())
@@ -302,6 +336,11 @@ int main(int argc, char *argv[])
             return 0;
         }
 
+        if (vm.count("list"))
+        {
+            bList = true;
+        }
+
         // Create lgogdownloader directories
         boost::filesystem::path path = Globals::globalConfig.sXMLDirectory;
         if (!boost::filesystem::exists(path))
@@ -472,6 +511,12 @@ int main(int argc, char *argv[])
             set_vm_value(vm, "threads", Globals::globalConfig.iThreads);
         }
 
+        if (Globals::globalConfig.iMsgLevel < -1)
+        {
+            Globals::globalConfig.iMsgLevel = -1;
+            set_vm_value(vm, "verbosity", Globals::globalConfig.iMsgLevel);
+        }
+
         Globals::globalConfig.curlConf.bVerifyPeer = !bInsecure;
         Globals::globalConfig.bColor = !bNoColor;
         Globals::globalConfig.bUnicode = !bNoUnicode;
@@ -539,6 +584,8 @@ int main(int argc, char *argv[])
         Globals::globalConfig.dlConf.bPatches = (Globals::globalConfig.dlConf.iInclude & OPTION_PATCHES);
         Globals::globalConfig.dlConf.bLanguagePacks = (Globals::globalConfig.dlConf.iInclude & OPTION_LANGPACKS);
         Globals::globalConfig.dlConf.bDLC = (Globals::globalConfig.dlConf.iInclude & OPTION_DLCS);
+
+        Globals::globalConfig.iListFormat = Util::getOptionValue(sListFormat, GlobalConstants::LIST_FORMAT, false);
     }
     catch (std::exception& e)
     {
@@ -578,15 +625,8 @@ int main(int argc, char *argv[])
     }
 
     // Make sure that directory has trailing slash
-    if (!Globals::globalConfig.dirConf.sDirectory.empty())
-    {
-        if (Globals::globalConfig.dirConf.sDirectory.at(Globals::globalConfig.dirConf.sDirectory.length()-1)!='/')
-            Globals::globalConfig.dirConf.sDirectory += "/";
-    }
-    else
-    {
-        Globals::globalConfig.dirConf.sDirectory = "./"; // Directory wasn't specified, use current directory
-    }
+    ensure_trailing_slash(Globals::globalConfig.dirConf.sDirectory, "./");
+    ensure_trailing_slash(Globals::globalConfig.dirConf.sWinePrefix, "./");
 
     // CA certificate bundle
     if (Globals::globalConfig.curlConf.sCACertPath.empty())
@@ -609,6 +649,8 @@ int main(int argc, char *argv[])
 
     // Init curl globally
     curl_global_init(CURL_GLOBAL_ALL);
+    struct CurlCleanup { ~CurlCleanup() { curl_global_cleanup(); } };
+    CurlCleanup _curl_cleanup;
 
     Downloader downloader;
 
@@ -636,7 +678,6 @@ int main(int argc, char *argv[])
     // Login failed, cleanup
     if (!bLoginOK && !bIsLoggedin)
     {
-        curl_global_cleanup();
         return 1;
     }
 
@@ -699,14 +740,12 @@ int main(int argc, char *argv[])
                 Util::setFilePermissions(Globals::globalConfig.sConfigFilePath, boost::filesystem::owner_read | boost::filesystem::owner_write);
             if (Globals::globalConfig.bSaveConfig)
             {
-                curl_global_cleanup();
                 return 0;
             }
         }
         else
         {
             std::cerr << "Failed to create config: " << Globals::globalConfig.sConfigFilePath << std::endl;
-            curl_global_cleanup();
             return 1;
         }
     }
@@ -719,13 +758,11 @@ int main(int argc, char *argv[])
             if (!Globals::globalConfig.bRespectUmask)
                 Util::setFilePermissions(Globals::globalConfig.sConfigFilePath, boost::filesystem::owner_read | boost::filesystem::owner_write);
 
-            curl_global_cleanup();
             return 0;
         }
         else
         {
             std::cerr << "Failed to create config: " << Globals::globalConfig.sConfigFilePath << std::endl;
-            curl_global_cleanup();
             return 1;
         }
     }
@@ -733,7 +770,6 @@ int main(int argc, char *argv[])
     bool bInitOK = downloader.init();
     if (!bInitOK)
     {
-        curl_global_cleanup();
         return 1;
     }
 
@@ -758,10 +794,8 @@ int main(int argc, char *argv[])
         downloader.repair();
     else if (Globals::globalConfig.bDownload) // Download games
         downloader.download();
-    else if (Globals::globalConfig.bListDetails || Globals::globalConfig.bList) // Detailed list of games/extras
+    else if (bList) // List games/extras/tags
         res = downloader.listGames();
-    else if (Globals::globalConfig.bListTags) // List tags
-        res = downloader.listTags();
     else if (!Globals::globalConfig.sOrphanRegex.empty()) // Check for orphaned files if regex for orphans is set
         downloader.checkOrphans();
     else if (Globals::globalConfig.bCheckStatus)
@@ -777,6 +811,39 @@ int main(int argc, char *argv[])
         }
         downloader.galaxyShowBuilds(product_id, build_index);
     }
+    else if (!galaxy_product_id_show_cloud_paths.empty())
+    {
+        int build_index = -1;
+        std::vector<std::string> tokens = Util::tokenize(galaxy_product_id_show_cloud_paths, "/");
+        std::string product_id = tokens[0];
+        if (tokens.size() == 2)
+        {
+            build_index = std::stoi(tokens[1]);
+        }
+        downloader.galaxyShowCloudSaves(product_id, build_index);
+    }
+    else if (!galaxy_product_id_show_local_cloud_paths.empty())
+    {
+        int build_index = -1;
+        std::vector<std::string> tokens = Util::tokenize(galaxy_product_id_show_local_cloud_paths, "/");
+        std::string product_id = tokens[0];
+        if (tokens.size() == 2)
+        {
+            build_index = std::stoi(tokens[1]);
+        }
+        downloader.galaxyShowLocalCloudSaves(product_id, build_index);
+    }
+    else if (!galaxy_product_cloud_saves_delete.empty())
+    {
+        int build_index = -1;
+        std::vector<std::string> tokens = Util::tokenize(galaxy_product_cloud_saves_delete, "/");
+        std::string product_id = tokens[0];
+        if (tokens.size() == 2)
+        {
+            build_index = std::stoi(tokens[1]);
+        }
+        downloader.deleteCloudSaves(product_id, build_index);
+    }
     else if (!galaxy_product_id_install.empty())
     {
         int build_index = -1;
@@ -788,6 +855,26 @@ int main(int argc, char *argv[])
         }
         downloader.galaxyInstallGame(product_id, build_index, Globals::globalConfig.dlConf.iGalaxyArch);
     }
+    else if (!galaxy_product_cloud_saves.empty()) {
+        int build_index = -1;
+        std::vector<std::string> tokens = Util::tokenize(galaxy_product_cloud_saves, "/");
+        std::string product_id = tokens[0];
+        if (tokens.size() == 2)
+        {
+            build_index = std::stoi(tokens[1]);
+        }
+        downloader.downloadCloudSaves(product_id, build_index);
+    }
+    else if (!galaxy_upload_product_cloud_saves.empty()) {
+        int build_index = -1;
+        std::vector<std::string> tokens = Util::tokenize(galaxy_upload_product_cloud_saves, "/");
+        std::string product_id = tokens[0];
+        if (tokens.size() == 2)
+        {
+            build_index = std::stoi(tokens[1]);
+        }
+        downloader.uploadCloudSaves(product_id, build_index);
+    }
     else
     {
         if (!Globals::globalConfig.bLogin)
@@ -802,7 +889,5 @@ int main(int argc, char *argv[])
     if (!Globals::globalConfig.sOrphanRegex.empty() && Globals::globalConfig.bDownload)
         downloader.checkOrphans();
 
-    curl_global_cleanup();
-
     return res;
 }
